import { IMaterialItem, IWebRuntimeAppData, IPlugin } from '../../../weapps-core';
import { BuildType, WebpackModeType, WebpackBuildCallBack } from '../../types/common';
import { RUNTIME } from '../../../types';
export interface IMpConfig {
    origin: string;
    entry: string;
    router: Record<string, string[]>;
    redirect: {
        notFound: string;
        accessDenied: string;
    };
    generate: {
        app: 'default' | 'noemit' | 'noconfig';
        appEntry?: string;
        subpackages?: Record<string, string[]>;
    };
    runtime: {
        wxComponent: 'default' | 'noprefix';
    };
    app: {
        navigationBarTitleText?: string;
    };
    projectConfig: {
        appid: string;
        projectname: string;
    };
    packageConfig: {
        author: string;
    };
}
export interface ICompileOpts {
    configPath: string;
    appBuildDir: string;
    appKey: string;
    generateMpType?: 'app' | 'subpackage';
    generateMpPath?: string;
    plugins?: IPlugin[];
}
export declare function startCompile(options: ICompileOpts, cb: WebpackBuildCallBack): void;
export declare function fixAppJson(appBuildDir: string): Promise<void>;
export declare function generateWebpackWebBuildParamsFile({ appId, allAppDataList, appBuildDir, materialsDir, dependencies, nodeModulesPath, publicPath, mode, watch, buildTypeList, assets, }: {
    appId: string;
    allAppDataList: IWebRuntimeAppData[];
    appBuildDir: string;
    materialsDir: string;
    dependencies: IMaterialItem[];
    nodeModulesPath: string;
    publicPath?: string;
    mode: WebpackModeType;
    watch: boolean;
    buildTypeList: BuildType[];
    assets: string[];
}): Promise<string>;
export interface IGenerateMpJsonConfigFileOpts {
    appKey?: string;
    generateMpType: 'app' | 'subpackage';
}
export declare function extractAndRemoveKbConfig(mainAppData: IWebRuntimeAppData, subAppDataList: IWebRuntimeAppData[], appBuildDir: string): Promise<{}>;
export declare function generateKboneTabBarConfig(mpConfig: Record<string, any>, appBuildDir: any): Promise<void>;
export declare function downloadAndWriteTabBarIcon(iconPath: any, pagePath: any, appBuildDir: string, fileName: string): Promise<unknown>;
export declare function generateKbonePageConfig(mpConfig: Record<string, any>, mainAppData: IWebRuntimeAppData, subAppDataList?: IWebRuntimeAppData[]): Record<string, any>;
export declare function generateKboneAppConfig(mpConfig: Record<string, any>, mainAppData: IWebRuntimeAppData): void;
export declare function generateMpJsonConfigFile(allAppDataList: IWebRuntimeAppData[], userConfig: any, appBuildDir: string, plugins: IPlugin[], options: IGenerateMpJsonConfigFileOpts): Promise<void>;
export declare function getMainAppDataByList(allAppDataList: IWebRuntimeAppData[]): IWebRuntimeAppData | undefined;
export declare function getHomePageInstance(pageInstanceList: any): any;
export declare function getPageName(name: string): string;
export declare function getMpAllRouterConfig(allAppDataList: IWebRuntimeAppData[], getHome?: boolean): any;
export declare function getWebpackWebBuildParams(appId: string, appBuildDir: string, materialsDir: string, dependencies: IMaterialItem[] | undefined, nodeModulesPath: string, publicPath?: string, mode?: WebpackModeType, watch?: boolean, buildTypeList?: BuildType[], extraDefine?: {}, assets?: string[]): any;
export declare function getWebpackMpBuildParams(appBuildDir: string, materialsDir: string, dependencies: IMaterialItem[] | undefined, nodeModulesPath: string, allAppDataList: IWebRuntimeAppData[], mode: string, watch: boolean, options: IGenerateMpJsonConfigFileOpts): any;
export declare function getAllPageMpEntryPath(allAppDataList: IWebRuntimeAppData[], appBuildDir: string, options?: IGenerateMpJsonConfigFileOpts): Record<string, any>;
export declare function downloadAndInstallDependencies(dependencies: IMaterialItem[] | undefined, materialsDir: string, installOptions?: IInstallOpts): Promise<void>;
export declare function downloadDependencies(targetDir: string, srcZipUrl: string): Promise<void>;
export interface IInstallOpts {
    packageName?: string;
    latest?: boolean;
    runtime?: RUNTIME;
    ignoreInstall?: boolean;
    isDependence?: boolean;
}
export declare function installDependencies(targetDir: string, options?: IInstallOpts): Promise<void>;
export declare function getMaterialNodeModulesPathList(dependencies: IMaterialItem[] | undefined, materialsDir: string): string[];
export declare function generateWebpackWebDevServerFile({ appBuildDir, buildTypeList, }: {
    appBuildDir: any;
    buildTypeList: any;
}): Promise<void>;
export declare function downloadAssets(targetDir: string, assetUrl: string): Promise<unknown>;
//# sourceMappingURL=webpack.d.ts.map