"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleMpPlugins = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const core_1 = require("./builder/core");
const lodash_1 = require("lodash");
function handleMpPlugins(plugins = [], appBuildDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const appBuildMpDir = path_1.default.resolve(appBuildDir, 'dist/mp');
        const appBuildNodeModulesDir = path_1.default.resolve(appBuildDir, 'node_modules');
        const mpBuildPkgJsonPath = path_1.default.resolve(appBuildMpDir, 'package.json');
        const buildPkgJson = fs_extra_1.default.readJsonSync(mpBuildPkgJsonPath);
        const sourcePkgJson = fs_extra_1.default.readJsonSync(path_1.default.join(appBuildDir, 'package.json'));
        plugins
            .filter((plugin) => plugin.type === 'mp')
            .forEach((plugin) => {
            const mpBuildAppJsonPath = path_1.default.resolve(appBuildMpDir, 'app.json');
            const pluginAppJsonPath = path_1.default.resolve(appBuildNodeModulesDir, plugin.module, 'app.json');
            mergeSubPackages(mpBuildAppJsonPath, pluginAppJsonPath);
            buildPkgJson.dependencies[plugin.module] =
                sourcePkgJson.dependencies[plugin.module];
        });
        fs_extra_1.default.writeJsonSync(mpBuildPkgJsonPath, buildPkgJson, { spaces: 2 });
        console.log('小程序安装依赖', appBuildMpDir);
        yield core_1.installDep(appBuildMpDir);
    });
}
exports.handleMpPlugins = handleMpPlugins;
function mergeSubPackages(baseAppJsonPath, mergeAppJsonPath) {
    const baseJson = fs_extra_1.default.readJSONSync(baseAppJsonPath);
    const mergeJson = fs_extra_1.default.readJSONSync(mergeAppJsonPath);
    if (!mergeJson.subpackages)
        return;
    const newJson = Object.assign({}, baseJson);
    if (!baseJson.subpackages) {
        newJson.subpackages = mergeJson.subpackages;
    }
    else {
        mergeJson.subpackages.forEach((mergeItem) => {
            const targetItemIdx = newJson.subpackages.findIndex((item) => {
                return item.root === mergeItem.root;
            });
            if (newJson.subpackages[targetItemIdx]) {
                const pages = lodash_1.uniq([].concat(newJson.subpackages[targetItemIdx].pages, mergeItem.pages));
                newJson.subpackages[targetItemIdx].pages = pages;
            }
            else {
                newJson.subpackages.push(mergeItem);
            }
        });
    }
    fs_extra_1.default.writeJSONSync(baseAppJsonPath, newJson, { spaces: 2 });
}
