"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateCodeFromTpl = exports.writeLowCodeFiles = exports.generateRouterFile = exports.getListenersString = exports.getComponentSchemaString = exports.isSlot = exports.pullComponentToListByInstance = exports.pullActionToListByInstances = exports.getOriginComponentAndActionList = exports.generateSinglePageJsxFile = exports.runGenerateCore = void 0;
const weapps_core_1 = require("../../weapps-core");
const path_1 = __importDefault(require("path"));
const template_1 = __importDefault(require("../template"));
const util_1 = require("../util");
const config_1 = require("../config");
const os_1 = __importDefault(require("os"));
const react_schema_renderer_1 = require("@formily/react-schema-renderer");
const style_1 = require("../util/style");
const lodash_template_1 = __importDefault(require("lodash.template"));
const chalk_1 = __importDefault(require("chalk"));
const types_1 = require("../../types");
const dataSource_1 = require("../../utils/dataSource");
const cals_1 = require("@cloudbase/cals");
function runGenerateCore(props) {
    return __awaiter(this, void 0, void 0, function* () {
        const { appBuildDir, appData, subAppDataList = [], dependencies = [], buildTypeList, appKey, basename, extraData = {
            isComposite: false,
            compProps: {},
        }, i18nConfig, fileCodeMap, isSandbox = false, deployMode, runtime = types_1.RUNTIME.NONE, ignoreInstall = false, } = props;
        const { domain } = appData.extra;
        const allAppDataList = [appData].concat(subAppDataList);
        const deps = {};
        allAppDataList.map((app) => {
            Object.entries(app.npmDependencies).forEach(([name, version]) => {
                deps[name] = version;
            });
        });
        dependencies.map((compLb) => {
            if (compLb.isComposite && compLb.compLibCommonResource) {
                Object.assign(deps, compLb.compLibCommonResource.npm);
            }
        });
        yield Promise.all(allAppDataList.map((data) => __awaiter(this, void 0, void 0, function* () {
            const { pageInstanceList, rootPath = '' } = data;
            console.log(chalk_1.default.blue.bold(`Generating files for ${rootPath ? 'Sub app ' + rootPath : 'Main app'}`));
            const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `packages/${rootPath}` : '');
            yield Promise.all(pageInstanceList.map((pageInstance) => generateSinglePageJsxFile(pageInstance, appBuildDir, rootPath, dependencies, extraData, fileCodeMap, isSandbox)));
            yield generateCodeFromTpl(appKey, data, dstDir, rootPath, extraData, i18nConfig, fileCodeMap, isSandbox, subAppDataList, buildTypeList, deployMode, domain);
            yield writeLowCodeFiles(data, dstDir, fileCodeMap, rootPath);
        })));
        yield generateRouterFile(allAppDataList, appBuildDir, basename, fileCodeMap, isSandbox);
    });
}
exports.runGenerateCore = runGenerateCore;
function getGenericCompFromDep(dependencies = []) {
    const genericCompMap = {};
    dependencies.forEach((compLib) => {
        if (compLib.isComposite) {
            compLib.components.forEach((component) => {
                util_1.deepDealComponentSchemaJson(Object.values(component.componentInstances)[0], (componentSchema) => {
                    if (componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.genericComp) {
                        genericCompMap[`${component.materialName}:${component.name}`] =
                            componentSchema.genericComp;
                    }
                });
            });
        }
    });
    return genericCompMap;
}
function generateSinglePageJsxFile(pageInstance, appBuildDir, rootPath, dependencies = [], extraData = {
    isComposite: false,
    compProps: {},
}, fileCodeMap, isSandbox) {
    return __awaiter(this, void 0, void 0, function* () {
        const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `packages/${rootPath}` : '');
        const { componentSchemaJson, data, style } = pageInstance;
        const genericCompMap = getGenericCompFromDep(dependencies);
        const { originComponentList, originActionList } = getOriginComponentAndActionList(componentSchemaJson, dependencies, [], [], genericCompMap);
        const componentInfo = yield util_1.getComponentsInfo(appBuildDir, dependencies, fileCodeMap);
        const { widgets, dataBinds, componentSchema } = getComponentSchemaString(componentSchemaJson, false, componentInfo);
        const pageClass = [rootPath, pageInstance.id].filter(Boolean).join('-');
        const templateData = {
            upperFirst: util_1.upperFirst,
            cleanVarName: util_1.cleanVarName,
            isSandbox,
            pageName: pageInstance.id,
            pageStyleText: weapps_core_1.toCssText(weapps_core_1.toCssStyle(style), `body`),
            useComponents: originComponentList,
            useActions: originActionList.filter((action) => action.type === weapps_core_1.ActionType.Material),
            componentSchema,
            widgets,
            dataBinds,
            pageClass,
            isComposite: extraData.isComposite,
            compProps: extraData.compProps,
            title: data.navigationBarTitleText || data.title || '',
        };
        const dest = path_1.default.join(dstDir, `/pages/${pageInstance.id}/index.jsx`);
        const template = template_1.default[`/src/pages/app.tpl`];
        const jsx = lodash_template_1.default(template.code, { interpolate: /<%=([\s\S]+?)%>/g })(templateData);
        fileCodeMap[dest] = {
            code: jsx,
        };
    });
}
exports.generateSinglePageJsxFile = generateSinglePageJsxFile;
function getOriginComponentAndActionList(componentSchema, dependencies, originComponentList = [], originActionList = [], genericCompMap = {}) {
    const fieldSchema = new react_schema_renderer_1.Schema(componentSchema);
    if (fieldSchema.isObject()) {
        const { 'x-props': xProps } = fieldSchema;
        if (xProps) {
            const { listenerInstances, sourceKey } = xProps;
            if (genericCompMap[sourceKey]) {
                const genericProps = genericCompMap[sourceKey];
                const genericComp = xProps.data[genericProps.propName];
                pullComponentToListByInstance(genericComp, originComponentList, dependencies);
            }
            pullComponentToListByInstance(sourceKey, originComponentList, dependencies);
            pullActionToListByInstances(listenerInstances, originActionList, dependencies);
        }
        const filedSchemaProperties = fieldSchema.properties || {};
        Object.keys(filedSchemaProperties).forEach((key) => {
            const schema = filedSchemaProperties[key];
            const schemaJson = schema;
            getOriginComponentAndActionList(schemaJson, dependencies, originComponentList, originActionList, genericCompMap);
        });
    }
    return {
        originComponentList,
        originActionList,
    };
}
exports.getOriginComponentAndActionList = getOriginComponentAndActionList;
function pullActionToListByInstances(listenerInstances, originActionList, fixedDependencies) {
    if (!listenerInstances || !listenerInstances.length) {
        return;
    }
    listenerInstances.map((pageListenerInstance) => {
        const { sourceKey, type } = pageListenerInstance;
        const { materialName, name, variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
        const material = fixedDependencies.find((m) => m.name === materialName);
        const actionKey = `${materialName}_${name}`;
        const isExistAction = originActionList.find((item) => item.key === actionKey);
        if (!isExistAction) {
            originActionList.push({
                name,
                materialName,
                materialVersion: material === null || material === void 0 ? void 0 : material.version,
                key: actionKey,
                type,
                variableName,
                entry: material === null || material === void 0 ? void 0 : material.entry,
            });
        }
    });
}
exports.pullActionToListByInstances = pullActionToListByInstances;
function pullComponentToListByInstance(sourceKey, originComponentList, fixedDependencies) {
    var _a, _b;
    const { materialName, name, variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
    const componentKey = `${materialName}_${name}`;
    const isExistComponent = originComponentList.find((item) => item.key === componentKey);
    if (!isExistComponent) {
        const foundOne = fixedDependencies.find((m) => m.name === materialName);
        if (!foundOne)
            return;
        const { schemaVersion = '' } = foundOne;
        let isPlainProps = false;
        try {
            if (Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0]) >= 3) {
                isPlainProps = true;
            }
        }
        catch (e) { }
        originComponentList.push({
            sourceKey,
            name: name || '',
            materialName: materialName || '',
            materialVersion: foundOne.version,
            isComposite: foundOne.isComposite,
            key: componentKey,
            variableName: variableName || '',
            isPlainProps,
            entry: (_b = foundOne) === null || _b === void 0 ? void 0 : _b.entry,
        });
    }
}
exports.pullComponentToListByInstance = pullComponentToListByInstance;
function isSlot(comp) {
    return comp.path && !comp['x-props'];
}
exports.isSlot = isSlot;
function getComponentSchemaString(componentSchema, isComposite = false, componentsInfoMap = {}, wrapperClass) {
    const componentInputProps = util_1.getInputProps(componentsInfoMap) || {};
    const copyJson = util_1.simpleDeepClone(componentSchema);
    const compWidgets = {};
    const compDataBinds = {};
    const componentSchemaJson = util_1.deepDealSchema(copyJson, (schema) => {
        var _a, _b, _c, _d, _e;
        const { 'x-props': xProps = {}, properties } = schema;
        const { dataBinds = [], commonStyle = {}, data = {}, classNameList = [], sourceKey, styleBind, classNameListBind, } = xProps;
        const componentInfo = componentsInfoMap[sourceKey];
        if ((_a = componentInfo) === null || _a === void 0 ? void 0 : _a.events) {
            schema['emitEvents'] = (_b = componentInfo) === null || _b === void 0 ? void 0 : _b.events.map((item) => item.name);
        }
        else if ((_c = componentInfo) === null || _c === void 0 ? void 0 : _c.emitEvents) {
            schema['emitEvents'] = componentInfo.emitEvents.map((item) => item.eventName);
        }
        if (!isSlot(schema) && schema.key) {
            const parentSchema = schema.parent;
            compWidgets[schema.key] = Object.assign(Object.assign({}, data), { style: weapps_core_1.toCssStyle(commonStyle), classList: classNameList, widgetType: sourceKey, _parentId: isSlot(parentSchema)
                    ? (_d = parentSchema.parent) === null || _d === void 0 ? void 0 : _d.key : parentSchema.key });
            if (dataBinds.length > 0) {
                compDataBinds[schema.key] = generateDataBinds(dataBinds, isComposite);
            }
            if (styleBind) {
                if (!styleBind.bindDataPath) {
                    console.warn('无 bindDataPath', xProps);
                }
                else {
                    styleBind.propertyPath = 'style';
                    compDataBinds[schema.key] = Object.assign(Object.assign({}, (compDataBinds[schema.key] || {})), generateDataBinds([styleBind], isComposite));
                }
            }
            if (classNameListBind) {
                classNameListBind.propertyPath = 'classList';
                compDataBinds[schema.key] = Object.assign(Object.assign({}, (compDataBinds[schema.key] || {})), generateDataBinds([classNameListBind], isComposite));
            }
        }
        delete xProps.data;
        delete xProps.dataTypes;
        delete xProps.dataBinds;
        delete xProps.styleBind;
        delete xProps.classNameListBind;
        if (properties && weapps_core_1.isEmptyObj(properties)) {
            delete schema.properties;
        }
        delete schema.type;
        if (xProps) {
            if (isComposite) {
                if (!((_e = schema.parent) === null || _e === void 0 ? void 0 : _e.parent)) {
                    if (!xProps['classNameList'])
                        xProps['classNameList'] = [];
                    xProps['classNameList'].push(wrapperClass);
                }
            }
            xProps['commonStyle'] = weapps_core_1.toCssStyle(xProps['commonStyle']);
            if (weapps_core_1.isEmptyObj(xProps['commonStyle'])) {
                delete xProps['commonStyle'];
            }
            if (weapps_core_1.isEmptyObj(xProps['style'])) {
                delete xProps['style'];
            }
            if (xProps['dataBinds'] && xProps['dataBinds'].length === 0) {
                delete xProps['dataBinds'];
            }
            if (xProps['listenerInstances'] &&
                xProps['listenerInstances'].length === 0) {
                delete xProps['listenerInstances'];
            }
            if (xProps['data']) {
                const xPropsData = xProps['data'];
                if (xPropsData._waFor && xPropsData._waFor.length === 0) {
                    delete xPropsData._waFor;
                }
                if (xPropsData.title === '') {
                    delete xPropsData.title;
                }
                if (weapps_core_1.isEmptyObj(xPropsData)) {
                    delete xProps['data'];
                }
            }
            if (xProps.listenerInstances) {
                xProps.listenerInstances = generateListnerInstances(xProps.listenerInstances, isComposite);
            }
            const syncProps = componentInputProps[xProps.sourceKey];
            if (syncProps) {
                if (!xProps.listenerInstances)
                    xProps.listenerInstances = [];
                Object.keys(syncProps).forEach((key) => {
                    let syncPropArr = [];
                    const syncProp = syncProps[key];
                    if (!Array.isArray(syncProp)) {
                        syncPropArr = [syncProp];
                    }
                    syncPropArr.forEach(({ changeEvent, valueFromEvent }) => {
                        xProps.listenerInstances.unshift({
                            trigger: changeEvent,
                            instanceFunction: `${config_1.REPLACE_SIGN}function({ event, forItems }) {
    const $for = forItems;
    const wid = ${isComposite ? 'this.widgets' : '$page.widgets'}.${schema.key};
    const widgetData = (forItems.forIndexes && forItems.forIndexes.length > 0) ? get(wid, forItems.forIndexes) : wid;
    widgetData.${key} = ${valueFromEvent};
  }.bind(this)${config_1.REPLACE_SIGN}`,
                        });
                    });
                });
            }
            if (xProps.dataBinds) {
                xProps.dataBinds = generateDataBinds(xProps.dataBinds, isComposite);
            }
            if (xProps.styleBind) {
                xProps.styleBind = generateDataBinds([xProps.styleBind], isComposite);
            }
            if (xProps.classNameListBind) {
                xProps.classNameListBind = generateDataBinds([xProps.classNameListBind], isComposite);
            }
        }
    });
    return {
        widgets: util_1.JsonToStringWithVariableName(compWidgets, { EOL: false }),
        dataBinds: util_1.JsonToStringWithVariableName(compDataBinds, { EOL: true }),
        componentSchema: util_1.JsonToStringWithVariableName(componentSchemaJson, {
            EOL: true,
        }),
    };
}
exports.getComponentSchemaString = getComponentSchemaString;
function generateDataBinds(dataBinds, isComposite) {
    const dataBindFuncs = {};
    dataBinds.forEach((bind) => {
        if (!bind.bindDataPath) {
            return console.warn('无 bindDataPath', bind.propertyPath);
        }
        let funcCode = '()=>{}';
        if (bind.type === weapps_core_1.PropBindType.forItem) {
            funcCode = `(forItems) => forItems.${bind.bindDataPath}`;
        }
        else if (bind.type === weapps_core_1.PropBindType.scope) {
            funcCode = `() => ({__type: "scopedValue", getValue: ($scope)=>$scope.${bind.bindDataPath}})}`;
        }
        else if (bind.type === weapps_core_1.PropBindType.expression) {
            if (isComposite) {
                funcCode = `(forItems) => {const $for = forItems; return (${bind.bindDataPath
                    .replace(/\$comp/g, 'this.$WEAPPS_COMP')
                    .replace(/;$/, '')})}`;
            }
            else {
                const code = bind.bindDataPath.replace(/;$/, '');
                funcCode = /\$scope\./.test(code)
                    ? `(forItems, event) => ({__type: "scopedValue", getValue: ($scope) => { const $for = forItems;return (${code})}})`
                    : `(forItems, event) => { const $for = forItems;return (${code})}`;
            }
        }
        else if (bind.type === weapps_core_1.PropBindType.prop) {
            let bindDataPath = bind.bindDataPath;
            const isNegated = bindDataPath.startsWith('!');
            if (isNegated)
                bindDataPath = bindDataPath.replace(/^!/, '');
            if (isComposite) {
                funcCode = `() => ${isNegated ? '!' : ''}${'this.$WEAPPS_COMP'}.props.data.${bindDataPath}`;
            }
            else {
                funcCode = `() => ${isNegated ? '!' : ''}$page.props.data.${bindDataPath}`;
            }
        }
        else {
            const PREFIX_MAP = {
                [weapps_core_1.PropBindType.state]: 'state',
                [weapps_core_1.PropBindType.computed]: 'computed',
                [weapps_core_1.PropBindType.dataVar]: 'dataVar',
                [weapps_core_1.PropBindType.stateData]: 'dataset.state',
                [weapps_core_1.PropBindType.paramData]: 'dataset.params',
            };
            switch (bind.type) {
                case weapps_core_1.PropBindType.state:
                case weapps_core_1.PropBindType.computed:
                case weapps_core_1.PropBindType.dataVar:
                case weapps_core_1.PropBindType.stateData:
                case weapps_core_1.PropBindType.paramData: {
                    if (bind.bindDataPath.startsWith('global.')) {
                        funcCode = bind.bindDataPath.replace(/^global./, `app.${PREFIX_MAP[bind.type]}.`);
                    }
                    else {
                        if (isComposite) {
                            funcCode = bind.bindDataPath
                                .replace(/^comp-\w+./, `this.$WEAPPS_COMP.${PREFIX_MAP[bind.type]}.`)
                                .replace(/^\$\w+_\d+./, `this.$WEAPPS_COMP.${PREFIX_MAP[bind.type]}.`);
                        }
                        else {
                            funcCode = bind.bindDataPath.replace(/^\w+./, `$page.${PREFIX_MAP[bind.type]}.`);
                        }
                    }
                    funcCode = `() => ${funcCode}`;
                    break;
                }
            }
        }
        dataBindFuncs[bind.propertyPath] = `${config_1.REPLACE_SIGN}${funcCode}${config_1.REPLACE_SIGN}`;
    });
    return dataBindFuncs;
}
function generateListnerInstances(listenerInstances, isComposite = false) {
    return listenerInstances.map((listener) => {
        var _a, _b, _c, _d, _e;
        const generatedListener = {
            key: listener.key,
            trigger: listener.trigger,
            isCapturePhase: listener.isCapturePhase,
            noPropagation: listener.noPropagation,
        };
        if (listener.type === weapps_core_1.ActionType.Material) {
            const { sourceKey } = listener;
            const { variableName } = util_1.getMetaInfoBySourceKey(sourceKey);
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}${variableName}${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.PropEvent) {
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({ data, event, forItems }) {
          event.detail.data = data;
          this.props.emit('${((_a = listener.handler) === null || _a === void 0 ? void 0 : _a.name) || listener.jsCode}', event.detail);
        }.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function(...args) { $page.props.events.${((_b = listener.handler) === null || _b === void 0 ? void 0 : _b.name) || listener.jsCode}.apply(null, args) }${config_1.REPLACE_SIGN}`;
            }
        }
        else if (listener.type === weapps_core_1.ActionType.Inline) {
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({event, domEvent, customEventData, data, forItems}) {
          return ${listener.jsCode}
        }.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({event, domEvent, customEventData, data, forItems}) {
          return ${listener.jsCode}
        }.bind(pageCodeContext)${config_1.REPLACE_SIGN}`;
            }
        }
        else if (listener.type === weapps_core_1.ActionType.Platform) {
            let name = ((_c = listener.handler) === null || _c === void 0 ? void 0 : _c.name) || listener.jsCode;
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { return app.${name}(data) }${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.DataSource) {
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { return app.cloud.callDataSource(data) }${config_1.REPLACE_SIGN}`;
        }
        else {
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}this.handler.${((_d = listener.handler) === null || _d === void 0 ? void 0 : _d.name) || listener.jsCode}.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                let name = ((_e = listener.handler) === null || _e === void 0 ? void 0 : _e.name) || listener.jsCode;
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}handler.${name}.bind(pageCodeContext)${config_1.REPLACE_SIGN}`;
            }
        }
        if (!weapps_core_1.isEmptyObj(listener.data)) {
            generatedListener.data = listener.data;
        }
        if (listener.dataBinds && listener.dataBinds.length > 0) {
            generatedListener.dataBinds = generateDataBinds(listener.dataBinds, isComposite);
        }
        return generatedListener;
    });
}
function getListenersString(listeners = [], isComposite = false) {
    return util_1.JsonToStringWithVariableName(generateListnerInstances(listeners, isComposite), {
        EOL: true,
    });
}
exports.getListenersString = getListenersString;
function generateRouterFile(allAppDataList, appBuildDir, basename = '', fileCodeMap, isSandbox) {
    return __awaiter(this, void 0, void 0, function* () {
        const routerImports = [];
        const routerRenders = [];
        const mountApis = [];
        let homePageId = '';
        yield Promise.all(allAppDataList.map((data) => __awaiter(this, void 0, void 0, function* () {
            const { pageInstanceList, rootPath = '' } = data;
            const pageFilePath = rootPath ? `/src/packages/${rootPath}/` : '/src/';
            pageInstanceList.map((pageInstance) => {
                const pageId = [rootPath, pageInstance.id].filter((i) => !!i).join('_');
                const pagePath = rootPath
                    ? `${rootPath}/${pageInstance.id}`
                    : `${pageInstance.id}`;
                if (pageInstance.isHome && !rootPath) {
                    homePageId = pageId;
                    routerRenders.push(`<Redirect from="/" exact to="/${pageId}"/>`);
                }
                if (pageInstance.isHome && rootPath) {
                    routerRenders.push(`<Redirect from="/${rootPath}" exact to="/${pagePath}"/>`);
                }
                if (isSandbox) {
                    routerImports.push(`import Page${pageId} from '${pageFilePath}pages/${pageInstance.id}/index';`);
                }
                else {
                    routerImports.push(`const Page${pageId} = React.lazy(() => import('${pageFilePath}pages/${pageInstance.id}/index'));`);
                }
                routerRenders.push(`<Route path="/${pagePath}" component={() => <Page${pageId} />}/>`);
            });
        })));
        const routerTemplate = template_1.default['/src/router/index.tpl'].code;
        const routerIndexStr = lodash_template_1.default(routerTemplate, {
            interpolate: /<%=([\s\S]+?)%>/g,
        })({
            routerImports: routerImports.join('\n'),
            routerRenders: routerRenders.join('\n'),
            mountApis: mountApis.join('\n'),
            basename: basename,
            homePageId,
            isHash: isSandbox,
        });
        const dest = path_1.default.resolve(appBuildDir, `src/router/index.jsx`);
        fileCodeMap[dest] = {
            code: routerIndexStr,
        };
    });
}
exports.generateRouterFile = generateRouterFile;
function writeLowCodeFiles(appData, appBuildDir, fileCodeMap, rootPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const lowcodeRootDir = path_1.default.join(appBuildDir, 'lowcode');
        const codeModules = util_1.generateDefaultLowcodeIndex(appData, 'global');
        const themeCode = codeModules.find((mod) => mod.type === cals_1.ECodeType.THEME);
        yield Promise.all(codeModules.map((m) => writeCode2file(m, 'global')));
        yield Promise.all(appData.pageInstanceList.map((page) => __awaiter(this, void 0, void 0, function* () {
            const codeModules = util_1.generateDefaultLowcodeIndex(page, page.id);
            yield codeModules.forEach((m) => writeCode2file(m, page.id, page));
        })));
        function writeCode2file(mod, pageId, page) {
            return __awaiter(this, void 0, void 0, function* () {
                const file = path_1.default.join(lowcodeRootDir, weapps_core_1.getCodeModuleFilePath(pageId, mod, {
                    style: '.css',
                }));
                let code = mod.code || '';
                let weappsApiPrefix = '';
                if (mod.type !== 'theme' && mod.type !== 'style') {
                    const isIncludeAppDot = !!code
                        .replace('通过 app.common.[name].xxx 访问这里定义的方法或值', '')
                        .match(/[^'"\w]?app[^'"\w]/g);
                    if (isIncludeAppDot) {
                        weappsApiPrefix = `import { app } from '${path_1.default
                            .relative(path_1.default.dirname(file), appBuildDir + '/app/global-api')
                            .replace(/\\/g, '/')}';`;
                    }
                }
                if (mod.type === 'style') {
                    let pageStyleString = '';
                    if (page) {
                        const pageClass = [rootPath, page.id].filter(Boolean).join('-');
                        pageStyleString = weapps_core_1.toCssText(weapps_core_1.toCssStyle(page.style), `.weapps-page-${pageClass}`);
                    }
                    code = yield style_1.processLess(((themeCode === null || themeCode === void 0 ? void 0 : themeCode.code) || style_1.defaultThemeCode) + code);
                }
                if (mod.type === 'theme') {
                    return;
                }
                fileCodeMap[file] = {
                    code: weappsApiPrefix + os_1.default.EOL + code.replace(/\$page/g, 'this.$page'),
                };
            });
        }
    });
}
exports.writeLowCodeFiles = writeLowCodeFiles;
function generateCodeFromTpl(appKey, appData, appBuildDir, rootPath, extraData, i18nConfig, fileCodeMap, isSandbox, subAppDataList, buildTypeList, deployMode, domain) {
    return __awaiter(this, void 0, void 0, function* () {
        const pageIds = [];
        const pageModules = {};
        appData.pageInstanceList.map((p) => {
            pageIds.push(p.id);
            pageModules[p.id] = p.codeModules;
        });
        const yyptConfig = yield util_1.getYyptConfigInfo(extraData);
        const templatesData = {
            'app/global-api.js': {
                appId: appKey,
                subPackageName: rootPath,
                domain: domain,
            },
            'app/handlers.js': {
                pageModules,
            },
            'app/common.js': {
                mods: appData.codeModules
                    .filter((m) => m.type === 'normal-module' && m.name !== '____index____')
                    .map((m) => m.name),
            },
            'store/computed.js': {
                pageIds,
            },
            'store/index.js': {},
            'utils/initGlobalVar.js': {},
            'datasources/index.js.tpl': {},
            'datasources/config.js.tpl': {
                appID: appKey,
                envID: appData.envId,
                isProd: deployMode === types_1.DEPLOY_MODE.UPLOAD,
            },
            'datasources/datasource-profiles.js.tpl': {
                datasourceProfiles: util_1.JsonToStringWithVariableName(dataSource_1.getDatasourceProfiles(appData.datasources || []), { EOL: true }),
            },
            'datasources/dataset-profiles.js.tpl': {
                datasetProfiles: util_1.JsonToStringWithVariableName(dataSource_1.getDatasetProfiles(appData, [appData]), { EOL: true }),
            },
        };
        if (!rootPath) {
            templatesData['index.jsx'] = Object.assign(Object.assign({}, yyptConfig), { i18nConfig,
                isSandbox, isBuildApp: buildTypeList.includes('app'), subAppDataList });
        }
        for (const file in templatesData) {
            const filePath = `/src/${file}`;
            if (!template_1.default[filePath]) {
                console.log('not found', filePath);
                process.exit();
            }
            const tplStr = template_1.default[filePath].code;
            const generatedCode = lodash_template_1.default(tplStr)(templatesData[file]);
            const outFile = path_1.default.resolve(appBuildDir, file.replace(/.tpl$/, ''));
            fileCodeMap[outFile] = {
                code: generatedCode,
            };
        }
    });
}
exports.generateCodeFromTpl = generateCodeFromTpl;
