"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDefaultLowcodeIndex = exports.cleanVarName = exports.camelcase = exports.upperFirst = exports.getFileNameByUrl = exports.writeLibCommonRes2file = exports.getYyptConfigInfo = exports.getInputProps = exports.getComponentsInfo = exports.deepDealSchema = exports.JsonToStringWithVariableName = exports.simpleDeepClone = exports.deepDeal = exports.isPlainObject = exports.isArray = exports.getMetaInfoBySourceKey = exports.deepDealComponentSchemaJson = exports.promiseWrapper = void 0;
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const react_schema_renderer_1 = require("@formily/react-schema-renderer");
const cals_1 = require("@cloudbase/cals");
const style_1 = require("./style");
function promiseWrapper(p) {
    return new Promise((resolve) => {
        try {
            p.then((i) => resolve([null, i])).catch((e) => resolve([e, null]));
        }
        catch (e) {
            resolve([e, null]);
        }
    });
}
exports.promiseWrapper = promiseWrapper;
function deepDealComponentSchemaJson(schema, dealFn) {
    if (!schema)
        return;
    dealFn(schema);
    if (schema.properties) {
        Object.keys(schema.properties).forEach((key) => {
            var _a;
            const childSchema = (_a = schema === null || schema === void 0 ? void 0 : schema.properties) === null || _a === void 0 ? void 0 : _a[key];
            if (childSchema) {
                deepDealComponentSchemaJson(childSchema, dealFn);
            }
        });
    }
}
exports.deepDealComponentSchemaJson = deepDealComponentSchemaJson;
function getMetaInfoBySourceKey(sourceKey) {
    if (!sourceKey) {
        return {};
    }
    const [materialName, name] = sourceKey.split(':');
    return {
        materialName,
        name,
        variableName: camelcase(materialName + '_' + name),
    };
}
exports.getMetaInfoBySourceKey = getMetaInfoBySourceKey;
function isArray(src) {
    return Object.prototype.toString.call(src) === '[object Array]';
}
exports.isArray = isArray;
function isPlainObject(src) {
    return Object.prototype.toString.call(src) === '[object Object]';
}
exports.isPlainObject = isPlainObject;
function deepDeal(src, reviver) {
    if (isArray(src)) {
        for (let i = 0, len = src.length; i < len; i++) {
            deepDeal(src[i], reviver);
            reviver === null || reviver === void 0 ? void 0 : reviver(i, src[i], src);
        }
    }
    if (isPlainObject(src)) {
        for (const key in src) {
            if (src.hasOwnProperty(key)) {
                deepDeal(src[key], reviver);
                reviver === null || reviver === void 0 ? void 0 : reviver(key, src[key], src);
            }
        }
    }
}
exports.deepDeal = deepDeal;
function simpleDeepClone(data) {
    return JSON.parse(JSON.stringify(data));
}
exports.simpleDeepClone = simpleDeepClone;
function JsonToStringWithVariableName(copyJson, options) {
    let variable = JSON.stringify(copyJson, null, 2)
        .replace(/"%%%(.*?)%%%"/g, function (match, expression) {
        return expression.replace(/\\"/g, '"');
    })
        .replace(/\\'/g, "'");
    if (options.EOL) {
        variable = variable.replace(/\\n/g, '\n').replace(/\\r/g, '\r');
    }
    return variable;
}
exports.JsonToStringWithVariableName = JsonToStringWithVariableName;
function deepDealSchema(sourceSchema, deal) {
    const fieldSchema = new react_schema_renderer_1.Schema(sourceSchema);
    if (fieldSchema === null || fieldSchema === void 0 ? void 0 : fieldSchema.isObject()) {
        Object.keys(fieldSchema.properties || {}).forEach((key) => {
            var _a, _b;
            const schema = (_a = fieldSchema.properties) === null || _a === void 0 ? void 0 : _a[key];
            const { 'x-props': xProps } = schema || {};
            if (((_b = xProps === null || xProps === void 0 ? void 0 : xProps['data']) === null || _b === void 0 ? void 0 : _b._visible) === false) {
                fieldSchema.setProperty(key, new react_schema_renderer_1.Schema({}));
            }
            else if (schema) {
                deepDealSchema(schema, deal);
                deal === null || deal === void 0 ? void 0 : deal(schema, key);
            }
        });
    }
    return fieldSchema.toJSON();
}
exports.deepDealSchema = deepDealSchema;
function getComponentsInfo(appBuildDir, dependencies, fileCodeMap) {
    return __awaiter(this, void 0, void 0, function* () {
        const outputObj = {};
        yield Promise.all(dependencies.map(({ name: materialName, components, isComposite }) => __awaiter(this, void 0, void 0, function* () {
            if (isComposite) {
                components.forEach((component) => {
                    let compItem = component;
                    const sourceKey = `${materialName}:${compItem.name}`;
                    outputObj[sourceKey] = Object.assign({ isComposite }, compItem);
                });
            }
            else {
                yield Promise.all(components.map(({ name, meta, emitEvents, evnets = undefined }) => __awaiter(this, void 0, void 0, function* () {
                    outputObj[materialName + ':' + name] = Object.assign(Object.assign({ isComposite }, meta), { events: evnets, emitEvents: emitEvents });
                })));
            }
        })));
        return outputObj;
    });
}
exports.getComponentsInfo = getComponentsInfo;
function getInputProps(componentsMetaMap) {
    const outputObj = {};
    for (let key in componentsMetaMap) {
        let component = componentsMetaMap[key];
        const sourceKey = key;
        if (component.isComposite) {
            let compItem = component;
            Object.keys(compItem.dataForm || {}).forEach((key) => {
                var _a, _b;
                const inputProps = ((_a = compItem.dataForm[key]) === null || _a === void 0 ? void 0 : _a.inputProp) || ((_b = compItem.dataForm[key]) === null || _b === void 0 ? void 0 : _b.syncProp);
                if (inputProps) {
                    outputObj[sourceKey] = Object.assign({ [key]: inputProps }, (outputObj[sourceKey] || {}));
                }
            });
        }
        else {
            const inputProps = (component === null || component === void 0 ? void 0 : component.inputProps) || (component === null || component === void 0 ? void 0 : component.syncProps);
            if (inputProps) {
                outputObj[sourceKey] = inputProps;
            }
        }
    }
    return outputObj;
}
exports.getInputProps = getInputProps;
function getYyptConfigInfo(extraData) {
    return __awaiter(this, void 0, void 0, function* () {
        const configJson = {
            yyptAppKey: '',
            reportUrl: '',
            stopReport: 'true',
        };
        if (!extraData || !extraData.operationService) {
            extraData = extraData || {};
            extraData.operationService = extraData.operationService || {
                yyptEnabled: true,
            };
        }
        const yyptAppKey = configJson.yyptAppKey || extraData.operationService.extAppId || '';
        const reportUrl = configJson.reportUrl || extraData.operationService.reportUrl || '';
        const stopReport = configJson.stopReport === 'true' || !extraData.operationService.yyptEnabled;
        return {
            yyptAppKey,
            reportUrl,
            stopReport,
        };
    });
}
exports.getYyptConfigInfo = getYyptConfigInfo;
function writeLibCommonRes2file(gItem, codeDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const compLibCommonResource = gItem.compLibCommonResource;
        const libCommonResFiles = [];
        libCommonResFiles.push({
            path: path_1.default.join(codeDir, `class.less`),
            code: `
        ${compLibCommonResource.theme.variable || ''}
        ${compLibCommonResource.class || ''}
        ${compLibCommonResource.theme.class || ''}
        `,
        }, {
            path: path_1.default.join(codeDir, 'const.js'),
            code: compLibCommonResource ? compLibCommonResource.const.code : '',
        }, {
            path: path_1.default.join(codeDir, 'tools.js'),
            code: compLibCommonResource ? compLibCommonResource.tools.code : '',
        });
        return libCommonResFiles.reduce((ret, item) => {
            ret[item.path] = {
                code: item.code,
            };
            return ret;
        }, {});
    });
}
exports.writeLibCommonRes2file = writeLibCommonRes2file;
function getFileNameByUrl(fileUrl) {
    const parsedUrl = url_1.default.parse(fileUrl);
    const filename = path_1.default.basename((parsedUrl === null || parsedUrl === void 0 ? void 0 : parsedUrl.pathname) || '');
    return filename;
}
exports.getFileNameByUrl = getFileNameByUrl;
function upperFirst(str) {
    return str[0].toUpperCase() + str.slice(1);
}
exports.upperFirst = upperFirst;
function camelcase(str, firstUpperCase = false) {
    str = str.replace(/[\:_-]([a-zA-Z])/g, function (l) {
        return l[1].toUpperCase();
    });
    if (firstUpperCase)
        str = str.charAt(0).toUpperCase() + str.slice(1);
    return str;
}
exports.camelcase = camelcase;
function cleanVarName(name) {
    return name.replace(/[@-]/g, '_');
}
exports.cleanVarName = cleanVarName;
function generateDefaultLowcodeIndex(data, id) {
    const lowCodes = (data === null || data === void 0 ? void 0 : data.lowCodes) || (data === null || data === void 0 ? void 0 : data.codeModules) || [];
    const fileTypes = [
        cals_1.ECodeType.STYLE,
        cals_1.ECodeType.STATE,
        cals_1.ECodeType.LIFECYCLE,
        cals_1.ECodeType.COMPUTEDN,
    ];
    if (id === 'global') {
        fileTypes.push(cals_1.ECodeType.THEME);
    }
    else if (id === '$comp') {
        fileTypes.push('index');
    }
    const EXIST_FILE_TYPE_MAP = {};
    lowCodes.forEach((mod) => {
        EXIST_FILE_TYPE_MAP[mod.type] = true;
    });
    fileTypes.forEach((type) => {
        const isHasLowcodeIndex = EXIST_FILE_TYPE_MAP[type];
        if (!isHasLowcodeIndex) {
            const code = {
                type,
                name: type,
                code: '',
                path: `${id}/${type}`,
            };
            switch (type) {
                case cals_1.ECodeType.STYLE: {
                    code.code = '';
                    break;
                }
                case cals_1.ECodeType.THEME: {
                    code.code = style_1.defaultThemeCode;
                    break;
                }
                case cals_1.ECodeType.STATE: {
                    code.code =
                        id === '$comp'
                            ? `export default function() {\n return {}\n}`
                            : `export default {}`;
                    break;
                }
                default: {
                    code.code = `export default {}`;
                }
            }
            lowCodes.unshift(code);
        }
    });
    return lowCodes.filter((m) => m.name !== '____index____');
}
exports.generateDefaultLowcodeIndex = generateDefaultLowcodeIndex;
