"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class NameMangler {
    constructor(opts = {}) {
        this.mangled = {};
        const leadingCharRanges = opts.leadingCharRanges || ['a', 'z', 'A', 'Z'];
        const candidateCharRanges = opts.candidateCharRanges || [
            'a',
            'z',
            'A',
            'Z',
            '0',
            '9',
        ];
        this.leadingChars = createCharsFromRanges(leadingCharRanges);
        this.candidateChars = createCharsFromRanges(candidateCharRanges);
        this.blackList = opts.blackList || [];
    }
    next() {
        if (!this.current) {
            this.current = [this.leadingChars[0]];
            return this.current.join('');
        }
        const len = this.current.length;
        for (let i = len - 1; i >= 0; i--) {
            const chars = i === 0 ? this.leadingChars : this.candidateChars;
            const curCharIdx = chars.indexOf(this.current[i]);
            if (curCharIdx < chars.length - 1) {
                this.current[i] = chars[curCharIdx + 1];
                break;
            }
            else if (i === 0) {
                this.current[i] = chars[0];
                this.current.push(chars[0]);
            }
            else {
                this.current[i] = chars[0];
            }
        }
        return this.current.join('');
    }
    mangle(name) {
        let mangled = this.mangled[name];
        if (!mangled) {
            mangled = this.next();
            if (this.blackList.indexOf(mangled) > -1) {
                return this.mangle(name);
            }
            this.mangled[name] = mangled;
        }
        return mangled;
    }
}
exports.default = NameMangler;
function createCharsFromRanges(range) {
    const chars = [];
    for (let i = 0; i + 1 < range.length; i += 2) {
        for (let code = range[i].charCodeAt(0); code <= range[i + 1].charCodeAt(0); code++) {
            chars.push(String.fromCharCode(code));
        }
    }
    return chars;
}
