import sdk from '@tcwd/weapps-sdk/lib/app-h5-sdk';
import { DS_SDK, CLOUD_SDK, createDataset, EXTRA_API } from '../datasources';
import { formatDate } from '../utils/date';
import { getter, setter, _isMobile } from '../utils';
import { scanCodeApi } from '../utils/scan-code-action';

export const subPackageName = '<%= subPackageName %>';
export const app = createGlboalApi();
mountAPIs(sdk);
function createGlboalApi() {
  const globalAPI = {
    id: '<%= appId %>',
    domain: '<%= domain %>',
    platform: 'WEB',

    formActions: {},
    pages: {},
    state: {},
    computed: {},
    common: {},
    // i18n,
    dataSources: DS_SDK,
    utils: {
      formatDate,
      get: getter,
      set: setter,
    },
  };

  const dataset = createDataset('$global');
  globalAPI.dataset = dataset;
  globalAPI.state.dataset = dataset;
  globalAPI.setState = (userSetState) => {
    Object.keys(userSetState).forEach((keyPath) => {
      globalAPI.utils.set(
        globalAPI.dataset.state,
        keyPath,
        userSetState[keyPath]
      );
    });
  };
  /**
   * 内部通用的设置状态变量值的方法
   *  varPath 结构为 $global.<变量名> 即全局变量
   *                $page.<变量名>  即当前页面变量
   *                <pageId>.<变量名> 指定页面 pageId 的变量 (应当避免修改非当前页面的变量值)
   */
  globalAPI._setStateVal = (config) => {
    EXTRA_API.setState(config.varPath, config.val);
  };
  // 给全局挂上 mainApp/subApp
  // The global api exposed to lowcode
  if (subPackageName) {
    globalAPI.mainApp = window[`$$app`];
    window[`$$subapp_${subPackageName}`] = globalAPI;
  } else {
    window['app'] = globalAPI;
    window[`$$app`] = globalAPI;
  }

  // 挂运营平台上报对象到app里
  globalAPI.yyptReport = window.yyptReport;

  // 预览区调试栏可调试
  // 暴露给复合组件，代码组件来引用
  window.app = window.app || globalAPI;

  // 避免被wx.cloud 覆盖
  globalAPI.cloud = CLOUD_SDK;
  return globalAPI;
}

export function createPageApi() {
  const $page = {
    state: {},
    computed: {},
    handler: {},
    props: {},
    widgets: {},
    // 页面数据源变量存储位置
    dataVar: {},
  };
  return $page;
}

// 分app 和 wx 挂载app
function mountAPIs(sdks) {
  Object.keys(sdks).forEach((item) => {
    let action = sdks[item];

    switch (item) {
      case 'showToast': {
        action = function (obj) {
          if (obj.icon === 'error' && !obj.image) {
            return sdks[item]({
              ...obj,
              image:
                'data:image/svg+xml,%3Csvg%20width%3D%22120%22%20height%3D%22120%22%20viewBox%3D%220,0,24,24%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%3Cpath%20d%3D%22M12%2010.586l5.657-5.657%201.414%201.414L13.414%2012l5.657%205.657-1.414%201.414L12%2013.414l-5.657%205.657-1.414-1.414L10.586%2012%204.929%206.343%206.343%204.93%2012%2010.586z%22%20fill-rule%3D%22evenodd%22%20fill%3D%22white%22%2F%3E%3C%2Fsvg%3E',
            });
          }
          return sdks[item](obj);
        };
        break;
      }
      case 'showModal': {
        const LIB_KEY = `@weapps-materials-main-gsd-h5-react`;
        const showModal =
          window[LIB_KEY] &&
          window[LIB_KEY].actions &&
          window[LIB_KEY].actions.showModal;

        if (!_isMobile() && showModal) {
          action = function (params) {
            return showModal({ data: params });
          };
        }
        break;
      }
      case 'scanCode': {
        action = (options) => {
          if (
            !options ||
            (!options.success && !options.fail && !options.complete)
          ) {
            return new Promise((resolve, reject) => {
              scanCodeApi({
                ...options,
                success: resolve,
                fail: reject,
              });
            });
          }
          scanCodeApi(options);
        };
        break;
      }
      case 'navigateTo':
      case 'reLaunch':
      case 'redirectTo': {
        action = function (obj) {
          const { url, ...restOpts } = obj;
          if (obj.mode === 'web') {
            if (item === 'navigateTo') {
              window.open(url);
            } else {
              window.location.href = url;
            }
          } else {
            return sdks[item]({
              ...restOpts,
              pageId: restOpts.pageId
                ? restOpts.pageId.replace(/^(\.)?\//, '')
                : restOpts.pageId,
            });
          }
        };
        break;
      }
    }
    app[item] = action;
  });
  return app;
}
